@@ -87,12 +87,20 @@ public class HeaderUtil {
 
     List<FileHeader> fileHeaders = zipModel.getCentralDirectory().getFileHeaders();
     if (indexOfFileHeader == fileHeaders.size() - 1) {
-      return getOffsetOfEndOfCentralDirectory(zipModel);
+      return getOffsetStartOfCentralDirectory(zipModel);
     } else {
       return fileHeaders.get(indexOfFileHeader + 1).getOffsetLocalHeader();
     }
   }
 
+  public static long getOffsetStartOfCentralDirectory(ZipModel zipModel) {
+    if (zipModel.isZip64Format()) {
+      return zipModel.getZip64EndOfCentralDirectoryRecord().getOffsetStartCentralDirectoryWRTStartDiskNumber();
+    }
+
+    return zipModel.getEndOfCentralDirectoryRecord().getOffsetOfStartOfCentralDirectory();
+  }
+
   public static List<FileHeader> getFileHeadersUnderDirectory(List<FileHeader> allFileHeaders, FileHeader rootFileHeader) {
     if (!rootFileHeader.isDirectory()) {
       return Collections.emptyList();
@@ -114,14 +122,6 @@ public class HeaderUtil {
     return totalUncompressedSize;
   }
 
-  private static long getOffsetOfEndOfCentralDirectory(ZipModel zipModel) {
-    if (zipModel.isZip64Format()) {
-      return zipModel.getZip64EndOfCentralDirectoryRecord().getOffsetStartCentralDirectoryWRTStartDiskNumber();
-    }
-
-    return zipModel.getEndOfCentralDirectoryRecord().getOffsetOfStartOfCentralDirectory();
-  }
-
   private static FileHeader getFileHeaderWithExactMatch(ZipModel zipModel, String fileName) throws ZipException {
     if (zipModel == null) {
       throw new ZipException("zip model is null, cannot determine file header with exact match for fileName: "
