@@ -18,6 +18,7 @@ package net.lingala.zip4j.headers;
 
 import net.lingala.zip4j.exception.ZipException;
 import net.lingala.zip4j.io.outputstream.CountingOutputStream;
+import net.lingala.zip4j.io.outputstream.OutputStreamWithSplitZipSupport;
 import net.lingala.zip4j.io.outputstream.SplitOutputStream;
 import net.lingala.zip4j.model.AESExtraDataRecord;
 import net.lingala.zip4j.model.ExtraDataRecord;
@@ -37,6 +38,7 @@ import java.nio.charset.Charset;
 import java.util.List;
 
 import static net.lingala.zip4j.util.FileUtils.getZipFileNameWithoutExtension;
+import static net.lingala.zip4j.util.InternalZipConstants.ZIP_64_NUMBER_OF_ENTRIES_LIMIT;
 import static net.lingala.zip4j.util.InternalZipConstants.ZIP_64_SIZE_LIMIT;
 import static net.lingala.zip4j.util.Zip4jUtil.isStringNotNullAndNotEmpty;
 
@@ -48,6 +50,7 @@ public class HeaderWriter {
 
   private RawIO rawIO = new RawIO();
   private byte[] longBuff = new byte[8];
+  private byte[] intBuff = new byte[4];
 
   public void writeLocalFileHeader(ZipModel zipModel, LocalFileHeader localFileHeader, OutputStream outputStream,
                                    Charset charset) throws IOException {
@@ -171,7 +174,7 @@ public class HeaderWriter {
 
     try(ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream()) {
       processHeaderData(zipModel, outputStream);
-      long offsetCentralDir = zipModel.getEndOfCentralDirectoryRecord().getOffsetOfStartOfCentralDirectory();
+      long offsetCentralDir = getOffsetOfCentralDirectory(zipModel);
       writeCentralDirectory(zipModel, byteArrayOutputStream, rawIO, charset);
       int sizeOfCentralDir = byteArrayOutputStream.size();
 
@@ -346,10 +349,10 @@ public class HeaderWriter {
 
   private void processHeaderData(ZipModel zipModel, OutputStream outputStream) throws IOException {
     int currentSplitFileCounter = 0;
-    if (outputStream instanceof CountingOutputStream) {
+    if (outputStream instanceof OutputStreamWithSplitZipSupport) {
       zipModel.getEndOfCentralDirectoryRecord().setOffsetOfStartOfCentralDirectory(
-          ((CountingOutputStream) outputStream).getFilePointer());
-      currentSplitFileCounter = ((CountingOutputStream) outputStream).getCurrentSplitFileCounter();
+          ((OutputStreamWithSplitZipSupport) outputStream).getFilePointer());
+      currentSplitFileCounter = ((OutputStreamWithSplitZipSupport) outputStream).getCurrentSplitFileCounter();
     }
 
     if (zipModel.isZip64Format()) {
@@ -360,6 +363,8 @@ public class HeaderWriter {
         zipModel.setZip64EndOfCentralDirectoryLocator(new Zip64EndOfCentralDirectoryLocator());
       }
 
+      zipModel.getZip64EndOfCentralDirectoryRecord().setOffsetStartCentralDirectoryWRTStartDiskNumber(
+          zipModel.getEndOfCentralDirectoryRecord().getOffsetOfStartOfCentralDirectory());
       zipModel.getZip64EndOfCentralDirectoryLocator().setNumberOfDiskStartOfZip64EndOfCentralDirectoryRecord(
           currentSplitFileCounter);
       zipModel.getZip64EndOfCentralDirectoryLocator().setTotalNumberOfDiscs(currentSplitFileCounter + 1);
@@ -389,11 +394,7 @@ public class HeaderWriter {
 
     try {
       final byte[] emptyShortByte = {0, 0};
-      boolean writeZip64ExtendedInfo = false;
-
-      if (isZip64Entry(fileHeader)) {
-        writeZip64ExtendedInfo = true;
-      }
+      boolean writeZip64ExtendedInfo = isZip64Entry(fileHeader);
 
       rawIO.writeIntLittleEndian(byteArrayOutputStream, (int) fileHeader.getSignature().getValue());
       rawIO.writeShortLittleEndian(byteArrayOutputStream, fileHeader.getVersionMadeBy());
@@ -446,10 +447,13 @@ public class HeaderWriter {
       }
       rawIO.writeShortLittleEndian(byteArrayOutputStream, fileCommentBytes.length);
 
-      //Skip disk number start for now
-      rawIO.writeShortLittleEndian(byteArrayOutputStream, fileHeader.getDiskNumberStart());
+      if (writeZip64ExtendedInfo) {
+        rawIO.writeIntLittleEndian(intBuff, 0, ZIP_64_NUMBER_OF_ENTRIES_LIMIT);
+        byteArrayOutputStream.write(intBuff, 0, 2);
+      } else {
+        rawIO.writeShortLittleEndian(byteArrayOutputStream, fileHeader.getDiskNumberStart());
+      }
 
-      //Skip internal file attributes for now
       byteArrayOutputStream.write(emptyShortByte);
 
       //External file attributes
@@ -539,7 +543,10 @@ public class HeaderWriter {
 
       rawIO.writeShortLittleEndian(outputStream, (int) extraDataRecord.getHeader());
       rawIO.writeShortLittleEndian(outputStream, extraDataRecord.getSizeOfData());
-      outputStream.write(extraDataRecord.getData());
+
+      if (extraDataRecord.getSizeOfData() > 0 && extraDataRecord.getData() != null) {
+        outputStream.write(extraDataRecord.getData());
+      }
     }
   }
 
@@ -660,6 +667,17 @@ public class HeaderWriter {
   private boolean isZip64Entry(FileHeader fileHeader) {
     return fileHeader.getCompressedSize() >= ZIP_64_SIZE_LIMIT
         || fileHeader.getUncompressedSize() >= ZIP_64_SIZE_LIMIT
-        || fileHeader.getOffsetLocalHeader() >= ZIP_64_SIZE_LIMIT;
+        || fileHeader.getOffsetLocalHeader() >= ZIP_64_SIZE_LIMIT
+        || fileHeader.getDiskNumberStart() >= ZIP_64_NUMBER_OF_ENTRIES_LIMIT;
+  }
+
+  private long getOffsetOfCentralDirectory(ZipModel zipModel) {
+    if (zipModel.isZip64Format()
+        && zipModel.getZip64EndOfCentralDirectoryRecord() != null
+        && zipModel.getZip64EndOfCentralDirectoryRecord().getOffsetStartCentralDirectoryWRTStartDiskNumber() != -1) {
+      return zipModel.getZip64EndOfCentralDirectoryRecord().getOffsetStartCentralDirectoryWRTStartDiskNumber();
+    }
+
+    return zipModel.getEndOfCentralDirectoryRecord().getOffsetOfStartOfCentralDirectory();
   }
 }
\ No newline at end of file
