@@ -1,6 +1,7 @@
 package net.lingala.zip4j.tasks;
 
 import net.lingala.zip4j.exception.ZipException;
+import net.lingala.zip4j.headers.HeaderUtil;
 import net.lingala.zip4j.headers.HeaderWriter;
 import net.lingala.zip4j.io.outputstream.SplitOutputStream;
 import net.lingala.zip4j.io.outputstream.ZipOutputStream;
@@ -10,8 +11,10 @@ import net.lingala.zip4j.model.ZipParameters;
 import net.lingala.zip4j.model.enums.CompressionMethod;
 import net.lingala.zip4j.model.enums.EncryptionMethod;
 import net.lingala.zip4j.progress.ProgressMonitor;
-import net.lingala.zip4j.tasks.RemoveEntryFromZipFileTask.RemoveEntryFromZipFileTaskParameters;
+import net.lingala.zip4j.tasks.RemoveFilesFromZipTask.RemoveFilesFromZipTaskParameters;
+import net.lingala.zip4j.util.BitUtils;
 import net.lingala.zip4j.util.FileUtils;
+import net.lingala.zip4j.util.InternalZipConstants;
 import net.lingala.zip4j.util.Zip4jUtil;
 
 import java.io.File;
@@ -20,9 +23,12 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.nio.charset.Charset;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 
 import static net.lingala.zip4j.headers.HeaderUtil.getFileHeader;
+import static net.lingala.zip4j.model.ZipParameters.SymbolicLinkAction.INCLUDE_LINK_AND_LINKED_FILE;
+import static net.lingala.zip4j.model.ZipParameters.SymbolicLinkAction.INCLUDE_LINK_ONLY;
 import static net.lingala.zip4j.model.enums.CompressionMethod.DEFLATE;
 import static net.lingala.zip4j.model.enums.CompressionMethod.STORE;
 import static net.lingala.zip4j.model.enums.EncryptionMethod.NONE;
@@ -31,6 +37,7 @@ import static net.lingala.zip4j.progress.ProgressMonitor.Task.ADD_ENTRY;
 import static net.lingala.zip4j.progress.ProgressMonitor.Task.CALCULATE_CRC;
 import static net.lingala.zip4j.progress.ProgressMonitor.Task.REMOVE_ENTRY;
 import static net.lingala.zip4j.util.CrcUtil.computeFileCrc;
+import static net.lingala.zip4j.util.FileUtils.assertFilesExist;
 import static net.lingala.zip4j.util.FileUtils.getRelativeFileName;
 import static net.lingala.zip4j.util.InternalZipConstants.BUFF_SIZE;
 import static net.lingala.zip4j.util.Zip4jUtil.javaToDosTime;
@@ -40,10 +47,12 @@ public abstract class AbstractAddFileToZipTask<T> extends AsyncZipTask<T> {
   private ZipModel zipModel;
   private char[] password;
   private HeaderWriter headerWriter;
+  private byte[] readBuff = new byte[BUFF_SIZE];
+  private int readLen = -1;
 
-  AbstractAddFileToZipTask(ProgressMonitor progressMonitor, boolean runInThread, ZipModel zipModel,
-                           char[] password, HeaderWriter headerWriter) {
-    super(progressMonitor, runInThread);
+  AbstractAddFileToZipTask(ZipModel zipModel, char[] password, HeaderWriter headerWriter,
+                           AsyncTaskParameters asyncTaskParameters) {
+    super(asyncTaskParameters);
     this.zipModel = zipModel;
     this.password = password;
     this.headerWriter = headerWriter;
@@ -52,40 +61,82 @@ public abstract class AbstractAddFileToZipTask<T> extends AsyncZipTask<T> {
   void addFilesToZip(List<File> filesToAdd, ProgressMonitor progressMonitor, ZipParameters zipParameters, Charset charset)
       throws IOException {
 
+    assertFilesExist(filesToAdd, zipParameters.getSymbolicLinkAction());
+
     List<File> updatedFilesToAdd = removeFilesIfExists(filesToAdd, zipParameters, progressMonitor, charset);
 
     try (SplitOutputStream splitOutputStream = new SplitOutputStream(zipModel.getZipFile(), zipModel.getSplitLength());
          ZipOutputStream zipOutputStream = initializeOutputStream(splitOutputStream, charset)) {
-      byte[] readBuff = new byte[BUFF_SIZE];
-      int readLen = -1;
 
       for (File fileToAdd : updatedFilesToAdd) {
         verifyIfTaskIsCancelled();
         ZipParameters clonedZipParameters = cloneAndAdjustZipParameters(zipParameters, fileToAdd, progressMonitor);
         progressMonitor.setFileName(fileToAdd.getAbsolutePath());
 
-        zipOutputStream.putNextEntry(clonedZipParameters);
-        if (fileToAdd.isDirectory()) {
-          zipOutputStream.closeEntry();
-          continue;
-        }
+        if (FileUtils.isSymbolicLink(fileToAdd)) {
+          if (addSymlink(clonedZipParameters)) {
+            addSymlinkToZip(fileToAdd, zipOutputStream, clonedZipParameters, splitOutputStream);
 
-        try (InputStream inputStream = new FileInputStream(fileToAdd)) {
-          while ((readLen = inputStream.read(readBuff)) != -1) {
-            zipOutputStream.write(readBuff, 0, readLen);
-            progressMonitor.updateWorkCompleted(readLen);
-            verifyIfTaskIsCancelled();
+            if (INCLUDE_LINK_ONLY.equals(clonedZipParameters.getSymbolicLinkAction())) {
+              continue;
+            }
           }
         }
 
-        FileHeader fileHeader = zipOutputStream.closeEntry();
-        fileHeader.setExternalFileAttributes(FileUtils.getFileAttributes(fileToAdd));
-
-        updateLocalFileHeader(fileHeader, splitOutputStream);
+        addFileToZip(fileToAdd, zipOutputStream, clonedZipParameters, splitOutputStream, progressMonitor);
       }
     }
   }
 
+  private void addSymlinkToZip(File fileToAdd, ZipOutputStream zipOutputStream, ZipParameters zipParameters,
+                               SplitOutputStream splitOutputStream) throws IOException {
+
+    ZipParameters clonedZipParameters = new ZipParameters(zipParameters);
+    clonedZipParameters.setFileNameInZip(replaceFileNameInZip(zipParameters.getFileNameInZip(), fileToAdd.getName()));
+    clonedZipParameters.setEncryptFiles(false);
+    clonedZipParameters.setCompressionMethod(CompressionMethod.STORE);
+
+    zipOutputStream.putNextEntry(clonedZipParameters);
+
+    String symLinkTarget = FileUtils.readSymbolicLink(fileToAdd);
+    zipOutputStream.write(symLinkTarget.getBytes());
+
+    closeEntry(zipOutputStream, splitOutputStream, fileToAdd, true);
+  }
+
+  private void addFileToZip(File fileToAdd, ZipOutputStream zipOutputStream, ZipParameters zipParameters,
+                            SplitOutputStream splitOutputStream, ProgressMonitor progressMonitor) throws IOException {
+
+    zipOutputStream.putNextEntry(zipParameters);
+
+    if (!fileToAdd.isDirectory()) {
+      try (InputStream inputStream = new FileInputStream(fileToAdd)) {
+        while ((readLen = inputStream.read(readBuff)) != -1) {
+          zipOutputStream.write(readBuff, 0, readLen);
+          progressMonitor.updateWorkCompleted(readLen);
+          verifyIfTaskIsCancelled();
+        }
+      }
+    }
+
+    closeEntry(zipOutputStream, splitOutputStream, fileToAdd, false);
+  }
+
+  private void closeEntry(ZipOutputStream zipOutputStream, SplitOutputStream splitOutputStream, File fileToAdd,
+                          boolean isSymlink) throws IOException {
+    FileHeader fileHeader = zipOutputStream.closeEntry();
+    byte[] fileAttributes = FileUtils.getFileAttributes(fileToAdd);
+
+    if (!isSymlink) {
+      // Unset the symlink byte if the entry being added is a symlink, but the original file is being added
+      fileAttributes[3] = BitUtils.unsetBit(fileAttributes[3], 5);
+    }
+
+    fileHeader.setExternalFileAttributes(fileAttributes);
+
+    updateLocalFileHeader(fileHeader, splitOutputStream);
+  }
+
   long calculateWorkForFiles(List<File> filesToAdd, ZipParameters zipParameters) throws ZipException {
     long totalWork = 0;
 
@@ -102,8 +153,7 @@ public abstract class AbstractAddFileToZipTask<T> extends AsyncZipTask<T> {
 
       //If an entry already exists, we have to remove that entry first and then add content again.
       //In this case, add corresponding work
-      String relativeFileName = getRelativeFileName(fileToAdd.getAbsolutePath(), zipParameters.getDefaultFolderPath(),
-          zipParameters.getRootFolderNameInZip());
+      String relativeFileName = getRelativeFileName(fileToAdd, zipParameters);
       FileHeader fileHeader = getFileHeader(getZipModel(), relativeFileName);
       if (fileHeader != null) {
         totalWork += (getZipModel().getZipFile().length() - fileHeader.getCompressedSize());
@@ -115,10 +165,7 @@ public abstract class AbstractAddFileToZipTask<T> extends AsyncZipTask<T> {
 
   ZipOutputStream initializeOutputStream(SplitOutputStream splitOutputStream, Charset charset) throws IOException {
     if (zipModel.getZipFile().exists()) {
-      if (zipModel.getEndOfCentralDirectoryRecord() == null) {
-        throw new ZipException("invalid end of central directory record");
-      }
-      splitOutputStream.seek(zipModel.getEndOfCentralDirectoryRecord().getOffsetOfStartOfCentralDirectory());
+      splitOutputStream.seek(HeaderUtil.getOffsetStartOfCentralDirectory(zipModel));
     }
 
     return new ZipOutputStream(splitOutputStream, password, charset, zipModel);
@@ -165,8 +212,7 @@ public abstract class AbstractAddFileToZipTask<T> extends AsyncZipTask<T> {
     clonedZipParameters.setLastModifiedFileTime(fileToAdd.lastModified());
 
     if (!Zip4jUtil.isStringNotNullAndNotEmpty(zipParameters.getFileNameInZip())) {
-      String relativeFileName = getRelativeFileName(fileToAdd.getAbsolutePath(), zipParameters.getDefaultFolderPath(),
-          zipParameters.getRootFolderNameInZip());
+      String relativeFileName = getRelativeFileName(fileToAdd, zipParameters);
       clonedZipParameters.setFileNameInZip(relativeFileName);
     }
 
@@ -198,8 +244,7 @@ public abstract class AbstractAddFileToZipTask<T> extends AsyncZipTask<T> {
     }
 
     for (File file : files) {
-      String fileName = getRelativeFileName(file.getAbsolutePath(), zipParameters.getDefaultFolderPath(),
-          zipParameters.getRootFolderNameInZip());
+      String fileName = getRelativeFileName(file, zipParameters);
 
       FileHeader fileHeader = getFileHeader(zipModel, fileName);
       if (fileHeader != null) {
@@ -217,10 +262,24 @@ public abstract class AbstractAddFileToZipTask<T> extends AsyncZipTask<T> {
     return filesToAdd;
   }
 
-  private void removeFile(FileHeader fileHeader, ProgressMonitor progressMonitor, Charset charset) throws ZipException {
-    RemoveEntryFromZipFileTask removeEntryFromZipFileTask = new RemoveEntryFromZipFileTask(progressMonitor, false,
-        zipModel);
-    removeEntryFromZipFileTask.execute(new RemoveEntryFromZipFileTaskParameters(fileHeader, charset));
+  void removeFile(FileHeader fileHeader, ProgressMonitor progressMonitor, Charset charset) throws ZipException {
+    AsyncTaskParameters asyncTaskParameters = new AsyncTaskParameters(null, false, progressMonitor);
+    RemoveFilesFromZipTask removeFilesFromZipTask = new RemoveFilesFromZipTask(zipModel, headerWriter, asyncTaskParameters);
+    removeFilesFromZipTask.execute(new RemoveFilesFromZipTaskParameters(Collections.singletonList(fileHeader.getFileName()), charset));
+  }
+
+  private String replaceFileNameInZip(String fileInZipWithPath, String newFileName) {
+    if (fileInZipWithPath.contains(InternalZipConstants.ZIP_FILE_SEPARATOR)) {
+      return fileInZipWithPath.substring(0, fileInZipWithPath.lastIndexOf(InternalZipConstants.ZIP_FILE_SEPARATOR) + 1) + newFileName;
+    }
+
+    return newFileName;
+  }
+
+
+  private boolean addSymlink(ZipParameters zipParameters) {
+    return INCLUDE_LINK_ONLY.equals(zipParameters.getSymbolicLinkAction()) ||
+        INCLUDE_LINK_AND_LINKED_FILE.equals(zipParameters.getSymbolicLinkAction());
   }
 
   @Override
