@@ -475,6 +475,16 @@ public class ExtractZipFileIT extends AbstractIT {
     ZipFileVerifier.verifyFileContent(TestUtils.getTestFileFromResources("öüäöäö/asöäööl"), outputFile);
   }
 
+  @Test
+  public void testExtractJarFileWithFileHeaderCompressedSize2() throws IOException {
+    extractFile(TestUtils.getTestArchiveFromResources("jar-dir-fh-entry-size-2.jar"));
+  }
+
+  @Test
+  public void testExtractJarFileWithOnlyFileAndLocalFileHeaderCompressedSize2() throws IOException {
+    extractFile(TestUtils.getTestArchiveFromResources("jar-dir-lfh-and-fh-entry-size-2.jar"));
+  }
+
   private void addFileToZip(ZipFile zipFile, String fileName, EncryptionMethod encryptionMethod, String password) throws ZipException {
     ZipParameters zipParameters = new ZipParameters();
     zipParameters.setEncryptFiles(encryptionMethod != null);
@@ -550,7 +560,9 @@ public class ExtractZipFileIT extends AbstractIT {
     return file.get();
   }
 
-  private File createZipFileAndSplit(List<File> filesToAddToZip, long splitLength, boolean encrypt, EncryptionMethod encryptionMethod) throws IOException {
+  private File createZipFileAndSplit(List<File> filesToAddToZip, long splitLength, boolean encrypt,
+                                     EncryptionMethod encryptionMethod) throws IOException {
+
     ZipFile zipFile = new ZipFile(generatedZipFile, PASSWORD);
     ZipParameters zipParameters = new ZipParameters();
     zipParameters.setEncryptFiles(encrypt);
@@ -560,4 +572,8 @@ public class ExtractZipFileIT extends AbstractIT {
     return TestUtils.splitFileWith7ZipFormat(zipFile.getFile(), temporaryFolder.getRoot(), splitLength);
   }
 
+  private void extractFile(File fileToExtract) throws IOException {
+    ZipFile zipFile = new ZipFile(fileToExtract);
+    zipFile.extractAll(outputFolder.getPath());
+  }
 }
