@@ -21,8 +21,10 @@ import java.io.OutputStream;
 import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Set;
 import java.util.stream.Collectors;
 
+import static net.lingala.zip4j.testutils.TestUtils.getTestFileFromResources;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.assertj.core.api.Assertions.fail;
 
@@ -435,6 +437,21 @@ public class MiscZipFileIT extends AbstractIT {
     }
   }
 
+  @Test
+  public void testZipSlipFixWithFileNameStartingWithParentDirectoryThrowsException() throws ZipException {
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setFileNameInZip("../somename.pdf");
+    zipFile.addFile(FILES_TO_ADD.get(0), zipParameters);
+
+    expectedException.expectMessage("illegal file name that breaks out of the target directory");
+    expectedException.expect(ZipException.class);
+
+    // Important here is that the name of the file in zip ("somename.pdf") should start with the
+    // name of the directory being extracted to ("some"). "somename.pdf" starts with "some".
+    zipFile.extractAll(outputFolder.getPath() + File.separator + "some");
+  }
+
   @Test
   public void testUnzipFileZipSlipWithNotNormalizedTarget() throws IOException {
     ZipFile zipFile = new ZipFile(generatedZipFile, PASSWORD);
@@ -459,6 +476,47 @@ public class MiscZipFileIT extends AbstractIT {
     }
   }
 
+  @Test
+  public void testCustomThreadFactory() throws IOException {
+    TestUtils.copyFileToFolder(getTestFileFromResources("file_PDF_1MB.pdf"), temporaryFolder.getRoot(), 1000);
+
+    String threadName = "CustomThreadFactoryTest";
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    zipFile.setThreadFactory(r -> {
+      Thread t = new Thread(threadName);
+      t.setDaemon(false);
+      return t;
+    });
+    zipFile.setRunInThread(true);
+
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setEncryptFiles(true);
+    zipParameters.setEncryptionMethod(EncryptionMethod.AES);
+
+    zipFile.addFolder(temporaryFolder.getRoot(), zipParameters);
+
+    Set<Thread> threadSet = Thread.getAllStackTraces().keySet();
+    List<Thread> zip4jThread = threadSet.stream().filter(e -> e.getName().equals(threadName)).collect(Collectors.toList());
+    assertThat(zip4jThread).hasSize(1);
+    assertThat(zip4jThread.get(0).getName()).isEqualTo(threadName);
+    assertThat(zip4jThread.get(0).isDaemon()).isFalse();
+  }
+
+  @Test
+  public void testGetExecutorServiceIsNullWhenNotInThreadMode() throws IOException {
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    zipFile.addFile(TestUtils.getTestFileFromResources("sample_text1.txt"));
+    assertThat(zipFile.getExecutorService()).isNull();
+  }
+
+  @Test
+  public void testGetExecutorServiceIsNotNullWhenInThreadMode() throws IOException {
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    zipFile.setRunInThread(true);
+    zipFile.addFile(TestUtils.getTestFileFromResources("sample_text1.txt"));
+    assertThat(zipFile.getExecutorService()).isNotNull();
+  }
+
   private void verifyInputStream(InputStream inputStream, File fileToCompareAgainst) throws IOException {
     File outputFile = temporaryFolder.newFile();
     try (OutputStream outputStream = new FileOutputStream(outputFile)) {
