@@ -2,6 +2,7 @@ package net.lingala.zip4j.util;
 
 import net.lingala.zip4j.exception.ZipException;
 import net.lingala.zip4j.model.ZipModel;
+import net.lingala.zip4j.model.ZipParameters;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.ExpectedException;
@@ -15,6 +16,7 @@ import java.nio.file.attribute.FileTime;
 import java.nio.file.spi.FileSystemProvider;
 import java.util.List;
 
+import static net.lingala.zip4j.util.InternalZipConstants.FILE_SEPARATOR;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.mockito.Mockito.doThrow;
 import static org.mockito.Mockito.mock;
@@ -139,6 +141,24 @@ public class FileUtilsTest {
     testGetFilesInDirectory(true, true, filesInDirectory, 3, false, false);
   }
 
+  @Test
+  public void testGetFileNameWithoutExtensionWithoutExtensionReturnsSameName() throws ZipException {
+    String fileNameWithoutExtension = FileUtils.getFileNameWithoutExtension("somename");
+    assertThat(fileNameWithoutExtension).isEqualTo("somename");
+  }
+
+  @Test
+  public void testGetFileNameWithoutExtensionReturnsJustTheName() throws ZipException {
+    String fileNameWithoutExtension = FileUtils.getFileNameWithoutExtension("somename.pdf");
+    assertThat(fileNameWithoutExtension).isEqualTo("somename");
+  }
+
+  @Test
+  public void testGetFileNameWithoutExtensionWithTwoExtensions() throws ZipException {
+    String fileNameWithoutExtension = FileUtils.getFileNameWithoutExtension("somename.pdf.001");
+    assertThat(fileNameWithoutExtension).isEqualTo("somename.pdf");
+  }
+
   @Test
   public void testGetZipFileNameWithoutExtensionThrowsExceptionWhenNull() throws ZipException {
     expectedException.expectMessage("zip file name is empty or null, cannot determine zip file name");
@@ -248,26 +268,59 @@ public class FileUtilsTest {
 
   @Test
   public void testGetRelativeFileNameWhenRootFoldersAreNull() throws ZipException {
-    assertThat(FileUtils.getRelativeFileName("somefile.txt", null, null)).isEqualTo("somefile.txt");
+    assertThat(FileUtils.getRelativeFileName(new File("somefile.txt"), new ZipParameters())).isEqualTo("somefile.txt");
   }
 
   @Test
   public void testGetRelativeFileWithRootFolderNameInZip() throws ZipException {
-    String expectedRootFolder = "rootfolder" + InternalZipConstants.FILE_SEPARATOR + "somefile.txt";
-    assertThat(FileUtils.getRelativeFileName("somefile.txt", null, "rootfolder")).isEqualTo(expectedRootFolder);
+    String expectetdFileName = "rootfolder" + FILE_SEPARATOR + "somefile.txt";
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setRootFolderNameInZip("rootfolder");
+    assertThat(FileUtils.getRelativeFileName(new File("somefile.txt"), zipParameters)).isEqualTo(expectetdFileName);
   }
 
   @Test
   public void testGetRelativeFileWithRootFolderNameInZipWithFileSeparator() throws ZipException {
-    String expectedRootFolder = "rootfolder" + InternalZipConstants.FILE_SEPARATOR + "somefile.txt";
-    assertThat(FileUtils.getRelativeFileName("somefile.txt", null, "rootfolder" + File.separator))
-        .isEqualTo(expectedRootFolder);
+    String expectetdFileName = "rootfolder" + FILE_SEPARATOR + "somefile.txt";
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setRootFolderNameInZip("rootfolder" + File.separator);
+    assertThat(FileUtils.getRelativeFileName(new File("somefile.txt"), zipParameters)).isEqualTo(expectetdFileName);
   }
 
   @Test
   public void testGetRelativeFileWithRootFolderNameInZipWithSeparatorsInName() throws ZipException {
-    String expectedRootFolder = "rootfolder" + InternalZipConstants.FILE_SEPARATOR + "somefile.txt";
-    assertThat(FileUtils.getRelativeFileName("somefile.txt", null, "rootfolder\\")).isEqualTo(expectedRootFolder);
+    String expectedRootFolder = "rootfolder" + FILE_SEPARATOR + "somefile.txt";
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setRootFolderNameInZip("rootfolder\\");
+    assertThat(FileUtils.getRelativeFileName(new File("somefile.txt"), zipParameters)).isEqualTo(expectedRootFolder);
+  }
+
+  @Test
+  public void testGetRelativeFileWithDifferentFileNameInZip() throws ZipException {
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setFileNameInZip("anotherName.txt");
+    assertThat(FileUtils.getRelativeFileName(new File("somefile.txt"), zipParameters)).isEqualTo("anotherName.txt");
+  }
+
+  @Test
+  public void testGetRelativeFileWithDifferentFileNameInZipAndRootFolderName() throws ZipException {
+    String expectedRootFolder = "rootfolder" + FILE_SEPARATOR + "anotherName.txt";
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setFileNameInZip("anotherName.txt");
+    zipParameters.setRootFolderNameInZip("rootfolder");
+    assertThat(FileUtils.getRelativeFileName(new File("somefile.txt"), zipParameters)).isEqualTo(expectedRootFolder);
+  }
+
+  @Test
+  public void testGetRelativeFileWithDifferentFileNameInZipAndRootFolderNameAndDefaultFolderPath() throws ZipException {
+    String expectedRootFolder = "rootfolder" + FILE_SEPARATOR + "anotherFolder" + FILE_SEPARATOR + "anotherName.txt";
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setFileNameInZip("anotherName.txt");
+    zipParameters.setDefaultFolderPath("defaultFolderPath");
+    zipParameters.setRootFolderNameInZip("rootfolder");
+    File fileToTest = new File("defaultFolderPath" + FILE_SEPARATOR + "anotherFolder" + FILE_SEPARATOR
+        + "somefile.txt");
+    assertThat(FileUtils.getRelativeFileName(fileToTest, zipParameters)).isEqualTo(expectedRootFolder);
   }
 
   @Test
@@ -285,6 +338,58 @@ public class FileUtilsTest {
     assertThat(FileUtils.isZipEntryDirectory("somename")).isFalse();
   }
 
+  @Test
+  public void testIsNumberedSplitFileForInvalidReturnsFalse() {
+    assertThat(FileUtils.isNumberedSplitFile(new File("somename.xyz"))).isFalse();
+  }
+
+  @Test
+  public void testIsNumberedSplitFileForValidSpltiFileReturnsTrue() {
+    assertThat(FileUtils.isNumberedSplitFile(new File("somename.xyz.001"))).isFalse();
+  }
+
+  @Test
+  public void testGetFileExtensionWithoutExtensionReturnsEmptyString() {
+    assertThat(FileUtils.getFileExtension(new File("somename"))).isEqualTo("");
+  }
+
+  @Test
+  public void testGetFileExtensionReturnsExtension() {
+    assertThat(FileUtils.getFileExtension(new File("somename.pdf"))).isEqualTo("pdf");
+  }
+
+  @Test
+  public void testGetFileExtensionWithDoubleExtensionReturnsLastExtension() {
+    assertThat(FileUtils.getFileExtension(new File("somename.pdf.001"))).isEqualTo("001");
+  }
+
+  @Test
+  public void testGetNextNumberedSplitFileCounterAsExtensionSingleDigit() {
+    assertThat(FileUtils.getNextNumberedSplitFileCounterAsExtension(8)).isEqualTo(".009");
+  }
+
+  @Test
+  public void testGetNextNumberedSplitFileCounterAsExtensionSkipsToDoubleDigit() {
+    assertThat(FileUtils.getNextNumberedSplitFileCounterAsExtension(9)).isEqualTo(".010");
+  }
+
+  @Test
+  public void testGetNextNumberedSplitFileCounterAsExtensionDoubleDigit() {
+    assertThat(FileUtils.getNextNumberedSplitFileCounterAsExtension(99)).isEqualTo(".100");
+  }
+
+  @Test
+  public void testGetNextNumberedSplitFileCounterAsExtensionThreeDigit() {
+    assertThat(FileUtils.getNextNumberedSplitFileCounterAsExtension(100)).isEqualTo(".101");
+  }
+
+  @Test
+  public void testIsSymbolicLinkWhenNoSuchMethodErrorReturnsFalse() {
+    File file = mock(File.class);
+    when(file.toPath()).thenThrow(NoSuchMethodError.class);
+    assertThat(FileUtils.isSymbolicLink(file)).isFalse();
+  }
+
   private File mockZipFileAsExists(String path, String zipFileNameWithoutExtension) {
     File file = mock(File.class);
     when(file.exists()).thenReturn(true);
